/*!
 * vue-siema v0.3.13
 * https://github.com/cannap/vue-siema
 *
 * @license
 * Copyright (c) 2016 Marko Bolliger <marko@renka.de> (https://renka.de)
 * Released under the MIT license
 * https://github.com/cannap/vue-siema/blob/master/LICENSE
 */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.Lib = factory());
}(this, (function () { 'use strict';

var mouseHandlers = {
  methods: {
    mousedownHandler: function mousedownHandler(e) {
      e.preventDefault();
      e.stopPropagation();
      this.pointerDown = true;
      this.drag.start = e.pageX;
    },
    mouseupHandler: function mouseupHandler(e) {
      e.stopPropagation();
      this.pointerDown = false;
      this.styleObject.cursor = '-webkit-grab';
      this.styleObject.transition = 'all ' + this.duration + 'ms ' + this.easing;
      if (this.drag.end) {
        this.updateAfterDrag();
      }

      this.clearDrag();
    },
    mousemoveHandler: function mousemoveHandler(e) {
      e.preventDefault();
      if (this.pointerDown) {
        this.drag.end = e.pageX;
        this.styleObject.cursor = '-webkit-grabbing';
        this.styleObject.transition = 'all 0ms ' + this.easing;
        this.styleObject.transform = 'translate3d(' + (this.currentSlide * (this.width / this.perPage) + (this.drag.start - this.drag.end)) * -1 + 'px, 0, 0)';
      }
    },
    mouseleaveHandler: function mouseleaveHandler(e) {
      if (this.pointerDown) {
        this.pointerDown = false;
        this.styleObject.cursor = '-webkit-grab';
        this.drag.end = e.pageX;
        this.styleObject.transition = 'all ' + this.duration + 'ms ' + this.easing;
        this.styleObject.webkitTransition = 'all ' + this.duration + 'ms ' + this.easing;
        this.updateAfterDrag();
        this.clearDrag();
      }
    }
  }
};

var Siema = { render: function render() {
    var _vm = this;var _h = _vm.$createElement;var _c = _vm._self._c || _h;return _c('div', { ref: "wrap", staticClass: "siema", on: { "mouseleave": _vm.mouseleaveHandler, "mouseup": _vm.mouseupHandler, "mousedown": _vm.mousedownHandler, "mousemove": _vm.mousemoveHandler, "touchstart": _vm.touchstartHandler, "touchend": _vm.touchendHandler, "touchmove": _vm.touchmoveHandler } }, [_c('div', { staticClass: "inner-siema", style: _vm.styleObject }, _vm._l(_vm.slides, function (slide) {
      return _c('div', { staticClass: "siema-slide", style: _vm.slideStyle, domProps: { "innerHTML": _vm._s(slide) } });
    }))]);
  }, staticRenderFns: [],
  mixins: [mouseHandlers],
  name: 'siema-slider',
  computed: {
    slideStyle: function slideStyle() {
      return {
        float: 'left',
        cssFloat: 'left',
        width: 100 / this.slides.length + '%'
      };
    }
  },

  props: {
    duration: {
      type: Number,
      default: 200
    },
    slides: {
      type: Array,
      required: true
    },
    easing: {
      type: String,
      default: 'ease-out'
    },
    perPage: {
      type: Number,
      default: 1
    },
    // Todo: find a beter name
    restartSliderAfterAdd: {
      type: Boolean,
      default: false
    },

    startIndex: {
      type: Number,
      default: 0
    },
    draggable: {
      type: Boolean,
      default: false
    },
    direction: {
      type: String,
      default: 'ltr'
    },
    threshold: {
      type: Number,
      default: 20
    },
    loop: {
      type: Boolean,
      default: true
    }
  },

  data: function data() {
    return {
      styleObject: {
        transform: 'none'
      },
      width: 0,
      currentSlide: 0,
      pointerDown: true,
      drag: {
        start: 0,
        end: 0
      }
    };
  },
  mounted: function mounted() {
    // Todo: Debounce

    if (this.draggable) {
      this.pointerDown = false;
      this.drag.start = 0;
      this.drag.end = 0;
    }
    // Fire
    this.init();
    window.addEventListener('resize', this.resize);
  },


  methods: {
    init: function init() {
      var siemaWidth = this.$refs.wrap.getBoundingClientRect().width;
      this.styleObject = Object.assign({}, this.styleObject, {
        width: siemaWidth / this.perPage * this.slides.length + 'px', // The Container width
        transition: 'all ' + this.duration + 'ms ' + this.easing,
        webkitTransition: 'all ' + this.duration + 'ms ' + this.easing,
        cursor: '-webkit-grab'
      });
      this.width = siemaWidth;
    },
    clearDrag: function clearDrag() {
      this.drag = {
        start: 0,
        end: 0
      };
    },
    updateAfterDrag: function updateAfterDrag() {
      var movement = this.drag.end - this.drag.start;
      if (movement > 0 && Math.abs(movement) > this.threshold) {
        this.prev();
      } else if (movement < 0 && Math.abs(movement) > this.threshold) {
        this.next();
      }
      this.slideToCurrent();
    },
    next: function next() {
      if (this.currentSlide === this.slides.length - this.perPage && this.loop) {
        this.currentSlide = 0;
      } else {
        this.currentSlide = Math.min(this.currentSlide + 1, this.slides.length - this.perPage);
      }
      this.slideToCurrent();
    },
    prev: function prev() {
      if (this.currentSlide === 0 && this.loop) {
        this.currentSlide = this.slides.length - this.perPage;
      } else {
        this.currentSlide = Math.max(this.currentSlide - 1, 0);
      }
      this.slideToCurrent();
    },
    slideToCurrent: function slideToCurrent() {
      this.styleObject.transform = 'translate3d(-' + this.currentSlide * (this.width / this.perPage) + 'px, 0, 0)';
    },
    resize: function resize() {
      var siemaWidth = this.$refs.wrap.getBoundingClientRect().width;
      this.styleObject.width = siemaWidth / this.perPage * this.slides.length + 'px';
      this.width = siemaWidth;
      this.slideToCurrent();
    },


    // Public

    goTo: function goTo(index) {
      this.currentSlide = Math.min(Math.max(index, 0), this.slides.length - 1);
      this.slideToCurrent();
    },
    touchstartHandler: function touchstartHandler(e) {
      e.stopPropagation();
      this.pointerDown = true;
      this.drag.start = e.touches[0].pageX;
    },
    touchendHandler: function touchendHandler(e) {
      e.stopPropagation();
      this.pointerDown = false;
      this.styleObject.transition = 'all ' + this.duration + 'ms ' + this.easing;
      if (this.drag.end) {
        this.updateAfterDrag();
      }
      this.clearDrag();
    },
    touchmoveHandler: function touchmoveHandler(e) {
      e.stopPropagation();
      if (this.pointerDown) {
        this.drag.end = e.touches[0].pageX;
        this.styleObject.transition = 'all 0ms ' + this.easing;
        this.styleObject.transform = 'translate3d(' + (this.currentSlide * (this.styleObject.width / this.perPage) + (this.drag.start - this.drag.end)) * -1 + 'px, 0, 0)';
      }
    }
  },

  beforeDestroy: function beforeDestroy() {
    window.removeEventListener('resize', this.resize);
  },


  watch: {
    'currentSlide': function currentSlide(newVal, oldVal) {
      // Todo: maybe save this in data for simpler reuse
      var callbackValues = {
        currentSlide: newVal + 1,
        isFirst: newVal === 0,
        isLast: newVal + 1 === this.slides.length
      };

      this.$emit('slideChange', callbackValues);
    },

    // Reinit the slider after new slides comes in
    'slides': function slides(newVal, oldVal) {
      this.init();
    }
  }
};

{
  console.log('Only print on development mode');
}

return Siema;

})));
